﻿using System;
using System.Diagnostics;

using Emgu.CV;
using Emgu.CV.Structure;
using System.Drawing;
#if !IOS
using Emgu.CV.GPU;
#endif

namespace TestProject
{
    /// <summary>
    /// 보행자 검출 헬퍼
    /// </summary>
    public static class PedestrianDetectionHelper
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 찾기 - Find(image, processingTime)

        /// <summary>
        /// 찾기
        /// </summary>
        /// <param name="image">이미지</param>
        /// <param name="processingTime">처리 시간</param>
        /// <returns>결과 사각형 배열</returns>
        public static Rectangle[] Find(Image<Bgr, Byte> image, out long processingTime)
        {
            Stopwatch   stopwatch;
            Rectangle[] resultRectangleArray;

            stopwatch = Stopwatch.StartNew();

#if !IOS
            if(GpuInvoke.HasCuda)
            {
                using(GpuHOGDescriptor descriptor = new GpuHOGDescriptor())
                {
                    descriptor.SetSVMDetector(GpuHOGDescriptor.GetDefaultPeopleDetector());

                    using(GpuImage<Bgr, Byte> bgrImage = new GpuImage<Bgr, byte>(image))
                    {
                        using(GpuImage<Bgra, Byte> bgraImage = bgrImage.Convert<Bgra, Byte>())
                        {
                            resultRectangleArray = descriptor.DetectMultiScale(bgraImage);
                        }
                    }
                }
            }
            else
#endif
            {
                using(HOGDescriptor descriptor = new HOGDescriptor())
                {
                    descriptor.SetSVMDetector(HOGDescriptor.GetDefaultPeopleDetector());

                    resultRectangleArray = descriptor.DetectMultiScale(image);
                }
            }

            stopwatch.Stop();

            processingTime = stopwatch.ElapsedMilliseconds;

            return resultRectangleArray;
        }

        #endregion
    }
}